@@ -55,9 +55,16 @@ void AuthBackend::setCapabilities(AuthBackend::Capabilities capabilities)
 
 bool AuthBackend::actionExists(const QString& action)
 {
+    Q_UNUSED(action);
     return false;
 }
 
+void AuthBackend::preAuthAction(const QString& action, QWidget* parent)
+{
+    Q_UNUSED(action)
+    Q_UNUSED(parent)
+}
+
 } //namespace KAuth
 
 #include "AuthBackend.moc"
